function [yout,yout_all]=raveevaluatenoisyfunction(src,d,ycol,data,yin,handles,ignore)
%evaluates the function for the columns in "ycol" in dataset "d" using the input data stored in the matrix
%"data".
%if ycol_sorted is a vector, evaluates all named functions and
%returns all responses as columns of a matrix in the order the appear in ycol_sorted, attempting to avoid calling the same function
%multiple times. All responses must share the same input data (data)
%
%5th argument, handles, is optional. Use it to reduce overhead of calling
%guidata.
%
%NOTE: this function does not enforce discrete variable values. That must be handled by whatever
%calls this.
%
%NOTE: if any of the inputs are strings, data will be a cell array instead
%of a double array.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

yout_all=cell(size(data,1),numel(ycol));

%handle empty cases:
if isempty(ycol);
    yout=zeros(size(data,1),0);
    return;
end
if size(data,1)==0
    yout=zeros(0,numel(ycol));
    return
end

if nargin<5
    handles=guidata(src);
end
prefs=handles.preferences{d};
targets=handles.targetvalues{d};

%deterine dependency:
[ycol_sorted,y_order]=sort(ycol,'descend');
f=numel(ycol_sorted);
while f<=numel(ycol_sorted)
    thesefunvars=handles.funvars{d}{ycol_sorted(f)};
    thesefunvars=setdiff(thesefunvars,ignore);
    ycol_sorted=union(ycol_sorted,thesefunvars(handles.isfunction{d}(thesefunvars)));
    f=f+1;
end

%Determine which varaibles are noise:
isnoisy=false(size(ycol_sorted));
for g=1:numel(ycol_sorted)
    index=ravegetindependentvars(handles,d,1,ycol_sorted(g),[]);
    if any(handles.isnoise{d}(index))
        isnoisy(g)=true;
    end
end

% Any non-noisy intermediate functions have already been evaluated by
%raveevaluatefunction. These are stored in yin. Append these values to data
%in the appropriate columns
ycol_noisy=ycol_sorted(isnoisy);
ycol_sorted=ycol_sorted(~isnoisy);

if any(ycol_sorted)
    data(ycol_sorted)=yin;
end
ycol_sorted=ycol_noisy;

%get unique rows:
[uniquedata,junk,rowidx]=unique(data,'rows');

name=handles.funnames{d}(ycol_sorted);
narg=handles.nargout{d}(ycol_sorted);

%% Which noise variables need to be sampled?
noisevars=thesefunvars(handles.isnoise{d}(thesefunvars));



%% Generate Monte Carlo Samples:

%Form Monte Carlo cases:
n=handles.prefs.randomsamplingsize;

i=handles.activedistribution{d}(noisevars);

method.name='rand';

%Do sampling:
mcpoints = ravesamplefromdistribution(src,method,d,1,noisevars,n,i);

%% Evaluate in batches:
maxbatch=handles.prefs.maxfunevalbatchsize;
M=size(uniquedata,1);

%Determine which rows of thisuniquedata will get evaluated in each batch:
if M*n<=maxbatch
    batchidx={1:M};
else
    batchsize=floor(maxbatch/n);
    nbatches=ceil(M/batchsize);
    lastbatchsize=rem(M,batchsize);
    if lastbatchsize==0
        lastbatchsize=batchsize;
    end
    mm=uint16(1):uint16(M);
    batchidx=mat2cell(mm,1,[batchsize+zeros(1,nbatches-1) lastbatchsize]);
end

yout=zeros(size(data,1),numel(ycol));

if nargout==2
    yout_all=zeros(size(data,1)*n,numel(ycol));
end


for q=1:numel(batchidx);
    b=batchidx{q};
    m=numel(b);
    
    thisuniquedata=uniquedata(b,:);
    
    % MAKE "FULL FACTORIAL" of thisuniquedata rows vs mcpoints
    thisuniquedata=repmat(thisuniquedata,[1,1,n]);
    thisuniquedata=permute(thisuniquedata,[3 1 2]);
    thisuniquedata=reshape(thisuniquedata,[m*n,size(uniquedata,2),1]);
    
    if q==1 || q==numel(batchidx); %repmats are slow, so only run these lines on the first and last iterations
        thismcpoints=repmat(mcpoints,[m,1]);
    end
    
    thisuniquedata(:,noisevars)=thismcpoints;
    
    
    %EVALUATE FUNCTIONS:
    if ~iscell(name) %one response only
        if handles.isfunction{d}(ycol_sorted)
            xfun=str2func(name);
            alloutputs=cell(1,nargout(xfun));
            
            switch handles.funclass{d}{ycol_sorted}
                case 'matlab_generic' %not used anymore but keep here for backward compatibility
                    if nargin(xfun)==3
                        [alloutputs{:}]=xfun(thisuniquedata,prefs,targets);
                    else
                        [alloutputs{:}]=xfun(thisuniquedata);
                    end
                case 'matlab_singleinput';
                    [alloutputs{:}]=xfun(thisuniquedata);
                case 'matlab_threeinput';
                    [alloutputs{:}]=xfun(thisuniquedata,prefs,targets);
                case 'matlab_varnames'; %each variable in its own column
                    udat=mat2cell(thisuniquedata(),size(thisuniquedata,1),ones(1,size(thisuniquedata,2)));
                    [alloutputs{:}]=xfun(udat{:});
                case 'independent'
                    alloutputs={thisuniquedata(:,ycol_sorted)};
                case 'bbnbnt' %bayesian network toolbox bbn
                    alloutputs=raveevaluatebbnbnt(src,d,ycol,data,handles);
            end
            
            y=alloutputs{narg};
        elseif ycol_sorted<=size(thisuniquedata,2)
            y=thisuniquedata(:,ycol_sorted);
        end
    else %multiple responses:
        %enlarge (if necessary) data to have as many columns as largest index:
        if size(thisuniquedata,2)<max(ycol_sorted)
            thisuniquedata=[thisuniquedata zeros(size(thisuniquedata,1),max(ycol_sorted)-size(thisuniquedata,2))];
        end
        
        isfuns=find(handles.isfunction{d}(ycol_sorted));
        if ~isempty(isfuns)
            
            [junk,index]=unique(name(isfuns)); %get index of first occurence of unique funs
            funname=name(isfuns);
            uniquefuns = funname(sort(index)); %extract unique funs without changing their evaluation order.
            
            for g=1:numel(uniquefuns);
                thisfun=uniquefuns{g};
                isthisfun=find(strcmp(name,thisfun)); %responses matching current function
                xfun=str2func(thisfun);
                alloutputs=cell(1,nargout(xfun));
                
                switch handles.funclass{d}{ycol_sorted(isfuns(g))}
                    case 'matlab_generic' %not used anymore but keep here for backward compatibility
                        if nargin(xfun)==3
                            [alloutputs{:}]=xfun(thisuniquedata,prefs,targets);
                        else
                            [alloutputs{:}]=xfun(thisuniquedata);
                        end
                    case 'matlab_singleinput';
                        [alloutputs{:}]=xfun(thisuniquedata);
                    case 'matlab_threeinput'; %
                        [alloutputs{:}]=xfun(thisuniquedata,prefs,targets);
                    case 'matlab_varnames'; %each variable in its own column
                        udat=thisuniquedata(:,handles.funvars{d}{ycol_sorted(isfuns(g))});
                        udat=mat2cell(udat,size(udat,1),ones(1,size(udat,2)));
                        [alloutputs{:}]=xfun(udat{:});
                    case 'independent'
                        alloutputs={thisuniquedata(:,ycol_sorted(isfuns(g)))};
                    case 'bbnbnt' %bayesian network toolbox bbn
                        alloutputs=raveevaluatebbnbnt(src,d,ycol,data,handles);
                end
                
                alloutputs=cell2mat(alloutputs);
                thisuniquedata(:,ycol_sorted(isthisfun))=alloutputs(:,narg(isthisfun));
                
                %y(:,isthisfun)=alloutputs(:,narg(isthisfun));
            end
            
        end
        y=thisuniquedata(:,ycol);
    end
    
    % ACCUMULATE MONTE CARLO RUNS INTO STATISTICS:
    stats=handles.aggregatingstatistic{d}(ycol);
    y_stat=accumulatestatistics(y,n,stats);
    
    [theserows,idx1]=ismember(rowidx,b);
    %     [theserows,idx2]=ismember(b,rowidx);
    
    yout(idx1>0,:)=y_stat(idx1(idx1>0),:);
    
    %Do we need to save the monte carlo data?
    if nargout==2
        %Deal out mc runs to appropriate places:
        left_idx=bsxfun(@plus,(find(idx1>0)-1).*n,(1:n))';
        right_idx=bsxfun(@plus,(idx1(idx1>0)-1).*n,(1:n))';
        
        yout_all(left_idx,:)=y(right_idx,:);
        
        
        %
        %         mcrowidx=bsxfun(@plus,(rowidx(ismember(rowidx,b))-1).*n,(1:n))';
        %         mcrowidx=mcrowidx(:);
        %         yout_all=[y(mcrowidx,:)];
    end
    
end

if nargout==2
    %reshape into 3d with MC along 3rd dimension
%     yout_all=permute(yout_all,[1 3 2]);
%     yout_all=reshape(yout_all,[],n,size(yout_all,3));
%     yout_all=permute(yout_all,[1,3,2]);
%     yout_all=num2cell(yout_all,[1 3]);
    
    yout_all=mat2cell(yout_all,n+zeros(size(data,1),1),ones(1,size(yout_all,2)));
    
    
    
end

function y_stat=accumulatestatistics(y,n,stats)
%Combine monte carlo runs into a single statistic in groups of n
%   size(y,1) must be divisible by n.

m=size(y,1)/n;

idx=repmat(1:m,n,1);
idx=idx(:);

for g=size(y,2):-1:1
    switch stats{g}{1}
        case 'mean'
            y_stat(:,g)=accumarray(idx,y(:,g),[],@mean);
        case 'median'
            y_stat(:,g)=accumarray(idx,y(:,g),[],@median);
        case 'stddev'
            y_stat(:,g)=accumarray(idx,y(:,g),[],@std);
        case 'variance'
            y_stat(:,g)=accumarray(idx,y(:,g),[],@var);
            
    end
end











