function h=ravehatchedline(xc,yc,theta,ar,spc,len,varargin)
%HATCHEDLINE Plot curve with hatched style
%   H=hatchedline(XC,YC,THETA,AR,SPC,LEN,VARARGIN) plots the curve 
%   specified in the vectors (XC, YC) with a hatched line style appropriate
%   for constriant diagrams.  VARARGIN should contain property/value pairs for line object.
%
%
%   As you move along the curve defined by xc/yc from 1:end the hashes will
%   be drawn on the right side of the curve (for positive theta). IE theta
%   is the clockwise rotation the the ticks are rotated through relative to
%   the direction of travel along the line.%
%
%
%   The tick angle (in radians) is specified in THETA.  When plotting curves
%   generated by contourc, positive THETA will point the hatches in the
%   decreasing direction of the contour variable.  If not specified,
%   THETA=45*pi/180 is used.
%
%   The aspect ratio of the plot window (y range divided by x range) is 
%   specified in AR.  If not specified, the current axes aspect ratio is
%   used.
%
%   The tick spacing is specified in SPC.  If positive, SPC is taken in
%   units of the x coordinate.  If negative, SPC is taken as a fraction of
%   the x range.  If not specified, -0.02 for 2% of x range is used.
%
%   The tick length (relative to the tick spacing) is specified in LEN.  If
%   not specified, SPC=1.4.
%
%   Any remaining options are passed to the plot command.  These may be
%   used to change line weights etc.
%
%   The graphics handles for all of the curves are returned in H.
%
%   See also HATCHEDCONTOURS, CONTOURC.

%   Rob McDonald 
%   ramcdona@calpoly.edu  
%   12 December 2006 v. 1.0
%   11 March 2007 v. 1.5 -- Rearranged inputs, added defaults for variable
%                           argument list.  Incompatible calling
%                           convention.
%
%
%   Modified slightly by Matt Daskilewicz 20 Nov 2008 to use "line" command instead
%   of plot. Also removed input handling cause that's how i like it. Used with permission.

%recommended values:
%theta=45*pi/180;
%ar=range of ylim/range of xlim for current graph;
%spc=-0.02;
%len=1.4;

% Done handling input options.

% 'dimensionalize' spc if specified nondimensional
if(spc < 0)
  ax=axis;
  spc=-spc*(ax(2)-ax(1));
end

% 'dimenionalize' length
len=spc*len;

% Find distance between points on the line
ds=((xc(2:end)-xc(1:end-1)).^2+((yc(2:end)-yc(1:end-1))/ar).^2).^0.5;

% Mask for elements of zero length
imask=(ds~=0);

% Eliminate duplicate points for interp1
ds=ds(imask);
xc=xc([imask true]);
yc=yc([imask true]);

% Build parametric coordinate along curve 
s=[0 cumsum(ds)];
stot=s(end);

% Pick parameter values for ticks. these are the values of s at which each
% tik will be placed along the curve.
stick=linspace(0,stot,ceil(stot/spc));

% Find points along the parameterized curve
%these are the x/y coordinates corresponding to the values of s_tick. 
xtick=interp1(s,xc,stick);
ytick=interp1(s,yc,stick);

% Find vectors in local direction of curve
u=(interp1(s,xc,stick+spc/100)-xtick)/(spc/ar); %componenet of vector in x direciton at each tick mark. 
v=(interp1(s,yc,stick+spc/100)-ytick)/spc; %component of vector in y direction at each tick mark
%vector [u v] points in same direction as curve at each point in stick. 

% Normalize slope into unit slope vector.
dr=(u.^2+v.^2).^0.5;
uv=[u./dr; v./dr];

ct=cos(theta);
st=sin(theta);
T=[ct -st;st ct];

% Rotate and scale unit vector into tick vector 
%this rotates each tick mark counterclockwise from the direction the curve is
%moving, 
dxy=(uv'*T)'*len;

H(1)=line(xc,yc,varargin{:}); %main constraint contour

xtickdata=zeros(3*numel(xtick),1);
ytickdata=zeros(3*numel(ytick),1);

for g=1:numel(xtick)
    xtickdata(3*(g-1)+1:3*g)=[xtick(g); xtick(g)+dxy(1,g);nan];
    ytickdata(3*(g-1)+1:3*g)=[ytick(g);ytick(g)+dxy(2,g)*ar;nan];
end

H(2)=line(xtickdata,ytickdata,varargin{:}); %tick marks

h=H;


